package net.sz.engine.utils;

import com.thoughtworks.xstream.XStream;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.zip.GZIPOutputStream;
import org.apache.log4j.Logger;

/**
 *
 *
 * @author 失足程序员
 * @mail 492794628@qq.com
 * @phone 13882122019
 */
public class XmlUtil {

    private static final Logger log = Logger.getLogger(XmlUtil.class);
    /*
     * XStream 不关心序列化/逆序列化的类的字段的可见性。
     * 序列化/逆序列化类的字段不需要 getter 和 setter 方法。
     * 序列化/逆序列化的类不需要有默认构造函数。
     */
    private static XStream xStream = new XStream();

    /**
     * obj -> xml(object类型转换为xml类型)
     *
     * @param obj
     * @return
     */
    public static String toXMLString(Object obj) {
        return xStream.toXML(obj);
    }

    /**
     * 将object类型转换为xml类型，并写入XML文件(其他格式也可以，比如txt文件)
     *
     * @param obj
     * @param path
     */
    public static void writerXML(Object obj, String path) {
        FileOutputStream fos = null;
        try {
            fos = new FileOutputStream(path);
            OutputStreamWriter osw = new OutputStreamWriter(fos, "UTF-8");
            String xml = toXMLString(obj);
            osw.write(xml);
            osw.flush();
        } catch (Exception ex) {
            log.error("序列化对象为xml错误：", ex);
        } finally {
            try {
                fos.close();
            } catch (IOException ex) {
            }
        }
    }

    /**
     * 读取XML文件，加载进相应Object类型
     *
     * @param <T>
     * @param path
     * @param t
     * @return
     */
    public static <T> T readerXMLToFile(String path, Class<T> t) {
        try {
            File file = new File(path);
            FileInputStream fis = new FileInputStream(file);
            BufferedInputStream bis = new BufferedInputStream(fis);
            int len = bis.available();
            byte[] data = new byte[len];
            bis.read(data);
            String xml = new String(data, "UTF-8");
            return readerXML(xml, t);
        } catch (Exception ex) {
            log.error("反序列化对象为xml错误：", ex);
        } finally {
        }
        return null;
    }

    /**
     * 读取XML文件，加载进相应Object类型
     *
     * @param <T>
     * @param xml
     * @param t
     * @return
     */
    public static <T> T readerXML(String xml, Class<T> t) {
        try {
            Object fromXML = xStream.fromXML(xml);
            return (T) fromXML;
        } catch (Exception ex) {
            log.error("反序列化对象为xml错误：", ex);
        } finally {
        }
        return null;
    }

    /**
     * 把xml数据写到zip文件中
     *
     * @param object
     * @param pathString
     * @throws Exception
     */
    public static void writerXMLZip(Object object, String pathString) throws Exception {
        //使用GZIPOutputStream包装OutputStream流，使其具体压缩特性，最后会生成test.txt.gz压缩包
        //并且里面有一个名为test.txt的文件
        BufferedOutputStream out = new BufferedOutputStream(new GZIPOutputStream(new FileOutputStream(pathString)));
        String toXMLString = toXMLString(object);
        byte[] bytes = toXMLString.getBytes("utf-8");
        out.write(bytes);
        out.close();
    }
}
